import numpy as np
cimport cython

@cython.boundscheck(False)
@cython.wraparound(False)
def cmix_cy(double [:,:] drodz, double [:, :] cij):
    
    cdef int n, nc, i, j, k
    nc = drodz.shape[0]
    n = drodz.shape[1]
    cdef double[:] suma = np.zeros(n)

    for k in range(n):
        for i in range(nc):
            for j in range(nc):
                suma[k] += cij[i,j]*drodz[i,k]*drodz[j,k]
    return suma.base


from __future__ import division 
import numpy as np
cimport cython


@cython.boundscheck(False)
@cython.wraparound(False)
def jcobi_roots(int n, int N0, int N1, int Al, int Be):
    
    cdef int i, j, k, Ab, Ad, Ap, nt
    cdef double z, zc, z1, y, x, xn, xn1, xd, xd1, xp, xp1
    cdef bint success
    #vectores derivadas y raices
    cdef double[:] dif1 = np.zeros(n)
    cdef double[:] dif2 = np.zeros(n)
    cdef double[:] root = np.zeros(n)
    
    #coeficientes polinomio Jacobi
    Ab = Al + Be
    Ad = Be - Al
    Ap = Be * Al
    
    dif1[0] =  (Ad/(Ab+2)+1)/2
    dif2[0] = 0.
    
    if not (n < 2):
        for i in range(1,n):
            z1 = i
            z = Ab + 2*z1
            dif1[i] = (Ab*Ad/z/(z+2)+1)/2
            if i == 1:
                dif2[i] = (Ab + Ap + z1)/z/z/(z+1)
            else:
                z **= 2
                y = z1 * (Ab + z1)
                y *= (Ap + y)
                dif2[i] = y / z / (z-1)
        
    #determinacion de las raices del polinomio de Jacobi
    
    x = 0.0
    for i in range(n):
        success = False
        while not success:
            xd = 0.0 
            xn = 1.0
            xd1 = 0.0
            xn1 = 0.0
            for j in range(n):
                xp = (dif1[j]-x)*xn - dif2[j]*xd
                xp1 = (dif1[j]-x)*xn1 - dif2[j]*xd1-xn
                xd = xn
                xd1 =  xn1
                xn = xp
                xn1 = xp1
            zc = 1.0
            z = xn/xn1
            if not (i == 0):
                for j in range(1,i+1):
                    zc -=  z / (x - root[j-1])
            z /= zc
            x -= z
            success = (abs(z) < 1e-9)
        root[i] = x
        x += 0.0001
        
    if N0 == 1 and N1 == 1:
        root = np.hstack([0,root,1])
    elif N0 == 1:
        root = np.hstack([0,root])
    elif N1 == 1:
        root = np.hstack([root,1])
    
    nt = n + N0 + N1
    dif1 = np.ones(nt)
    
    for k in range(nt):
        x=root[k]
        for j in range(nt):
            if j!=k:
                y = x-root[j]
                dif1[k] = y*dif1[k]
    
    return root.base, dif1.base


@cython.boundscheck(False)
@cython.wraparound(False)
def colocAB(double [:] roots):
    cdef int n, k, j, i 
    cdef double x, y, gen
    n = roots.shape[0]
    cdef double [:] dif1 = np.ones(n)
    cdef double [:] dif2 = np.zeros(n)
    cdef double [:] dif3 = np.zeros(n)
    cdef double [:,:] B = np.zeros([n,n])
    cdef double [:,:] A = np.zeros([n,n])
    for k in range(n):
        x=roots[k]
        for j in range(n):
            if j!=k:
                y=x-roots[j]
                dif3[k]=y*dif3[k]+3.*dif2[k]
                dif2[k]=y*dif2[k]+2.*dif1[k]
                dif1[k]=y*dif1[k]
    for i in range(n):
        for j in range(n):
            if j!=i:
                y=roots[i]-roots[j]
                gen=dif1[i]/dif1[j]/y
                A[i,j]=gen
                B[i,j]=gen*(dif2[i]/dif1[i]-2/y)
            else:
                A[i,j]=dif2[i]/dif1[i]/2
                B[i,j]=dif3[i]/dif1[i]/3
    return A.base, B.base

@cython.boundscheck(False)
@cython.wraparound(False)
@cython.cdivision(True)
def colocA(double [:] roots):
    cdef int n, k, j, i 
    cdef double x, y, gen
    n = roots.shape[0]
    cdef double [:] dif1 = np.ones(n)
    cdef double [:] dif2 = np.zeros(n)
    cdef double [:,:] A = np.zeros([n,n])
    for k in range(n):
        x=roots[k]
        for j in range(n):
            if j!=k:
                y=x-roots[j]
                dif2[k]=y*dif2[k]+2.*dif1[k]
                dif1[k]=y*dif1[k]
    for i in range(n):
        for j in range(n):
            if j!=i:
                y=roots[i]-roots[j]
                gen=dif1[i]/dif1[j]/y
                A[i,j]=gen
            else:
                A[i,j]=dif2[i]/dif1[i]/2
    return A.base

@cython.boundscheck(False)
@cython.wraparound(False)
def colocB(double [:] roots):
    cdef int n, k, j, i 
    cdef double x, y, gen
    n = roots.shape[0]
    cdef double [:] dif1 = np.ones(n)
    cdef double [:] dif2 = np.zeros(n)
    cdef double [:] dif3 = np.zeros(n)
    cdef double [:,:] B = np.zeros([n,n])
    for k in range(n):
        x=roots[k]
        for j in range(n):
            if j!=k:
                y=x-roots[j]
                dif3[k]=y*dif3[k]+3.*dif2[k]
                dif2[k]=y*dif2[k]+2.*dif1[k]
                dif1[k]=y*dif1[k]
    for i in range(n):
        for j in range(n):
            if j!=i:
                y=roots[i]-roots[j]
                gen=dif1[i]/dif1[j]/y
                B[i,j]=gen*(dif2[i]/dif1[i]-2/y)
            else:
                B[i,j]=dif3[i]/dif1[i]/3
    return B.base



from __future__ import division, print_function, absolute_import
import numpy as np


def lngmie(ghs, g1s, g2s, be, be2):

    lng = np.log(ghs) + (be*g1s+be2*g2s)/ghs
    return lng


def dlngmie_drho(ghs, g1s, g2s, be, be2):

    ghs, dghs = ghs

    g1, dg1 = g1s
    g2, dg2 = g2s

    aux = (be*g1+be2*g2)

    lng = np.log(ghs) + aux/ghs

    dlng = be * ghs * (dg1 + be * dg2)
    dlng += dghs * (ghs - aux)
    dlng /= ghs**2
    return np.hstack([lng, dlng])


def d2lngmie_drho(ghs, g1s, g2s, be, be2):

    ghs, dghs, d2ghs = ghs
    g1, dg1, d2g1 = g1s
    g2, dg2, d2g2 = g2s

    ghs2 = ghs**2
    ghs3 = ghs2 * ghs
    dghs2 = dghs**2
    aux1 = be*g1+be2*g2
    aux2 = be*dg1+be2*dg2

    lng = np.log(ghs) + aux1/ghs

    dlng = ghs * aux2
    dlng += dghs * (ghs - aux1)
    dlng /= ghs2

    d2lng = 2*dghs2*aux1
    d2lng += ghs2*(d2ghs + be * (d2g1+be*d2g2))
    d2lng += ghs*(-2*dghs*aux2-dghs2 - aux1*d2ghs)
    d2lng /= ghs3

    return np.hstack([lng, dlng, d2lng])



from setuptools import setup, Extension
# from Cython.Distutils import build_ext

try:
    from Cython.Distutils import build_ext
except ImportError:
    use_cython = False
else:
    use_cython = True

"""
cmdclass = {}
ext_modules = []


if use_cython:
    ext_modules += [Extension('sgtpy.coloc_cy',
                              ['sgtpy/src/coloc_cy.pyx']),
                    Extension('sgtpy.sgt.cijmix_cy',
                              ['sgtpy/src/cijmix_cy.pyx'])]
    cmdclass.update({'build_ext': build_ext})
else:
    ext_modules += [Extension('sgtpy.coloc_cy', ['sgtpy/src/coloc_cy.c']),
                    Extension('sgtpy.sgt.cijmix_cy',
                    ['sgtpy/src/cijmix_cy.c'])]
"""

cmdclass = {}
ext_modules = []


ext_modules += [Extension('sgtpy.coloc_cy',
                          ['sgtpy/src/coloc_cy.pyx']),
                Extension('sgtpy.sgt.cijmix_cy',
                          ['sgtpy/src/cijmix_cy.pyx'])]
cmdclass.update({'build_ext': build_ext})


setup(
  name='sgtpy',
  license='MIT',
  version='0.0.22',
  description='SAFT-VR-MIE EOS and SGT',
  author='Gustavo Chaparro Maldonado, Andres Mejia Matallana, Erich A. Muller',
  author_email='g.chaparro-maldonado21@imperial.ac.uk',
  url='https://github.com/gustavochm/sgtpy',
  download_url='https://github.com/gustavochm/sgtpy.git',
  long_description=open('long_description.rst').read(),
  packages=['sgtpy', 'sgtpy.vrmie_mixtures',  'sgtpy.vrmie_pure',
            'sgtpy.gammamie_mixtures', 'sgtpy.gammamie_pure',
            'sgtpy.sgt', 'sgtpy.equilibrium', 'sgtpy.fit'],
  cmdclass=cmdclass,
  ext_modules=ext_modules,
  package_data={'sgtpy': ['database/*']},
  install_requires=['numpy', 'scipy', 'cython', 'numba', 'pandas', 'openpyxl'],
  platforms=["Windows", "Linux", "Mac OS", "Unix"],
  keywords=['SAFT-VR-Mie', 'SGT'],
  zip_safe=False
)



from __future__ import division, print_function, absolute_import
import numpy as np


def g2mca(xhi00, khs, xm, da2new, suma_g2, eps, a1vdw_cte):

    g2 = 3 * da2new / eps
    g2 -= khs * suma_g2 / xhi00
    g2 /= xm
    g2 /= - a1vdw_cte

    return g2


def dg2mca_dxhi00(xhi00, khs, dkhs, xm, d2a2new, dsuma_g2, eps, a1vdw_cte):

    sum1, dsum1 = dsuma_g2
    g2 = 3. * np.asarray(d2a2new) / eps

    g2[0] -= khs * sum1 / xhi00
    g2[1] += khs * sum1 / xhi00**2
    g2[1] -= (sum1 * dkhs + khs * dsum1)/xhi00
    g2 /= xm
    g2 /= - a1vdw_cte
    return g2


def d2g2mca_dxhi00(xhi00, khs, dkhs, d2khs, xm, d3a2new,
                   d2suma_g2, eps, a1vdw_cte):

    sum1, dsum1, d2sum1 = d2suma_g2

    g2 = 3. * np.asarray(d3a2new) / eps

    aux1 = khs * sum1 / xhi00
    aux2 = aux1 / xhi00
    aux3 = (sum1 * dkhs + khs * dsum1)/xhi00

    g2[0] -= aux1
    g2[1] += aux2
    g2[1] -= aux3
    g2[2] += 2*aux3/xhi00
    g2[2] -= 2*aux2/xhi00
    g2[2] -= (2*dkhs*dsum1 + sum1*d2khs + khs*d2sum1)/xhi00
    g2 /= xm
    g2 /= - a1vdw_cte

    return g2


def dg2mca_dx(xhi00, khs, dkhsx, xm, ms, da2new, da2newx, suma_g2, suma_g2x,
              eps, a1vdw_cte):

    g2 = 3 * da2new / eps
    g2 -= khs * suma_g2 / xhi00
    g2 /= xm
    g2 /= - a1vdw_cte

    dg2x = -np.multiply.outer(dkhsx,  suma_g2)
    dg2x -= khs * suma_g2x
    dg2x += khs * np.multiply.outer(ms, suma_g2) / xm
    dg2x /= xhi00
    dg2x -= 3 * np.multiply.outer(ms, da2new / eps) / xm
    dg2x += 3. * da2newx / eps
    dg2x /= xm
    dg2x /= -a1vdw_cte

    return g2, dg2x


def dg2mca_dxxhi(xhi00, khs, dkhs, dkhsx, xm, ms, d2a2new, da2newx,
                 dsuma_g2, suma_g2x, eps, a1vdw_cte):

    sum1, dsum1 = dsuma_g2
    g2 = 3. * np.asarray(d2a2new) / eps

    g2[0] -= khs * sum1 / xhi00
    g2[1] += khs * sum1 / xhi00**2
    g2[1] -= (sum1 * dkhs + khs * dsum1)/xhi00
    g2 /= xm
    g2 /= - a1vdw_cte

    dg2x = -np.multiply.outer(dkhsx,  sum1)
    dg2x -= khs * suma_g2x
    dg2x += khs * np.multiply.outer(ms, sum1) / xm
    dg2x /= xhi00
    dg2x -= 3 * np.multiply.outer(ms, d2a2new[0] / eps) / xm
    dg2x += 3. * da2newx / eps
    dg2x /= xm
    dg2x /= -a1vdw_cte
    return g2, dg2x
